

from fastapi import FastAPI, HTTPException, UploadFile, File
from imghst.app.configuration.configuration import Configuration
from loguru import logger
import aiofiles
from datetime import datetime
import typer
import filetype

from .request_models import (
    HealthCheckRequest,
    NewImageRequestResponse
)

app = FastAPI()
app.configuration_object = Configuration()


@app.get("/health", response_model=HealthCheckRequest)
async def health_check() -> HealthCheckRequest:
    return HealthCheckRequest(
        systemStatus=True
    )


@app.post("/uploadImage/{apiKey}", response_model=NewImageRequestResponse)
async def upload_image_to_file_path(apiKey, image_upload: UploadFile = File(...)) -> NewImageRequestResponse:
    
    if apiKey != app.configuration_object.api_request_key:
        raise HTTPException(status_code=400, detail="Invalid API Key.")

    current_configuration_image_path = app.configuration_object.image_hosting_folder
    
    if not current_configuration_image_path.exists():
        raise HTTPException(status_code=500, detail="Improper configuration was set. The current configuration image hosting folder does not exist.")
    

    _supported_file_types = [
        "image/jpeg",
        "image/jpx",
        "image/png",
        "image/gif",
        "image/webp",
        "image/x-canon-cr2",
        "image/tiff",
        "image/bmp",
        "image/vnd.ms-photo",
        "image/vnd.adobe.photoshop",
        "image/x-icon",
        "image/heic"
    ]

    if image_upload.content_type not in _supported_file_types:
        await image_upload.file.close()
        raise HTTPException(status_code=400, detail="Unsupported file type uploaded.")
    
    
    # check if valid.
    try:
        detect_file_type = filetype.guess(await image_upload.read())
        if detect_file_type is None:
            raise HTTPException(status_code=400, detail="Unknown file type was uploaded.")

        if detect_file_type.mime != image_upload.content_type:
            raise HTTPException(status_code=500, detail="Mismatch content type header and mime value of object.")
    except Exception:
        logger.exception("Unknown error occurred for detecting file type.")
        raise HTTPException(status_code=500, detail="System error on detecting file type.")

    generate_a_unique_id = app.configuration_object.image_unique_file_name()
    
    async with aiofiles.open(str(current_configuration_image_path / f"{generate_a_unique_id}.{detect_file_type.extension}"), "wb") as opened_file:
        await image_upload.seek(0)
        await opened_file.write(await image_upload.read())
        image_upload.file.close()

    return NewImageRequestResponse(
        api_status=True,
        date_processed=datetime.now().isoformat(),
        file_direct_path_name=str(current_configuration_image_path / f"{generate_a_unique_id}.{detect_file_type.extension}")
    )



import typer
import uvicorn

from imghst.app.app import app
from imghst.__version__ import Version
from imghst.app.configuration.configuration import Configuration
from pathlib import Path

cli_app = typer.Typer()



@cli_app.command()
def run(port: int = Configuration.port_number_to_run_on, directory_path: str = Configuration.image_hosting_folder, api_key: str = Configuration.api_request_key):
    
    splash_text = f"""
  _                 _         _   
 (_)_ __ ___   __ _| |__  ___| |_ 
 | | '_ ` _ \ / _` | '_ \/ __| __|
 | | | | | | | (_| | | | \__ \ |_ 
 |_|_| |_| |_|\__, |_| |_|___/\__|
              |___/               
                    v{Version()}
    """
    
    print(splash_text)
    print("\n")
    current_configuration_object = Configuration()
    current_configuration_object.api_request_key = api_key
    current_configuration_object.port_number_to_run_on = portNumber
    current_configuration_object.image_hosting_folder = Path(directory_path)
    

    app.configuration_object = current_configuration_object
    uvicorn.run(app, port=current_configuration_object.port_number_to_run_on)


if __name__ == "__main__":
    cli_app()




from imghst.app.configuration.unique_generators.uuid_based import generate_unique_id
from pathlib import Path

class Configuration:
    """ Do Not Touch Anything Here. Configure with arguments. """
    
    api_request_key = "DEFAULTCONFIGURATION"
    image_unique_file_name = generate_unique_id
    image_hosting_folder = Path(".")
    
    port_number_to_run_on = 5000





from setuptools import find_packages, setup
from imghst.__version__ import Version


setup(
    name="imghst",
    version=str(Version()),
    author="Mustafa Mohamed",
    author_email="git@ms7m.me",
    description="A simple and fast image hoster for applications like ShareX.",
    long_description="A simple and fast image hoster for applications like ShareX.",
    long_description_content_type="text/markdown",
    url="https://github.com/ms7m/imghst",
    python_requires=">=3.6.0",
    packages=find_packages(
        exclude=["testing", "*.testing", "*.testing.*", "testing.*"]
    ),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": [
            "imghst = imghst.entry:cli_app"
        ]
    },
    include_package_data=True,
    install_requires=open("./requirements.txt", 'r').readlines())




class Version:
    major = 0
    minor = 2
    patch = 0
    
    alpha_build = False
    beta_build = False
    developer_build = False

    def __str__(self):
        _human_readable_version = f"{self.major}.{self.minor}.{self.patch}"
        if self.alpha_build:
            _human_readable_version += ".alpha"

        if self.beta_build:
            _human_readable_version += ".beta"

        if self.developer_build:
            _human_readable_version += ".developer-build"

        return _human_readable_version



from datetime import datetime
from typing import Any

from pydantic import BaseModel
from pydantic.class_validators import validator

from .configuration.configuration import Configuration


class NewImageRequestResponse(BaseModel):
    """ Response model after uploading a new image """

    api_status: bool
    date_processed: datetime
    file_direct_path_name: str

    
class HealthCheckRequest(BaseModel):
    """ Request model for health check endpoint """
    
    systemStatus: bool

